/* valatype.vala
 *
 * Copyright (C) 2006  Raffaele Sandrini
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Raffaele Sandrini <rasa@gmx.ch>
 */
#define VALA_FREE_CHECKED(o,f) ((o) == NULL ? NULL : ((o) = (f (o), NULL)))
#define VALA_FREE_UNCHECKED(o,f) ((o) = (f (o), NULL))

#include "valaarray.h"
#include <vala/valaarray.h>
#include <vala/valareport.h>
#include <vala/valasourcereference.h>

struct _ValaArrayPrivate {
	ValaDataType* _element_type;
	int _rank;
	char* cname;
};
#define VALA_ARRAY_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_ARRAY, ValaArrayPrivate))
enum  {
	VALA_ARRAY_DUMMY_PROPERTY,
	VALA_ARRAY_ELEMENT_TYPE,
	VALA_ARRAY_RANK,
};
static char* vala_array_real_get_cname (ValaDataType* base, gboolean const_type);
static gboolean vala_array_real_is_reference_type (ValaDataType* base);
static char* vala_array_real_get_upper_case_cname (ValaDataType* base, char* infix);
static char* vala_array_real_get_lower_case_cname (ValaDataType* base, char* infix);
static char* vala_array_real_get_free_function (ValaDataType* base);
static GList* vala_array_real_get_cheader_filenames (ValaDataType* base);
static char* vala_array_real_get_marshaller_type_name (ValaDataType* base);
static char* vala_array_real_get_get_value_function (ValaDataType* base);
static char* vala_array_real_get_set_value_function (ValaDataType* base);
static GObject * vala_array_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer vala_array_parent_class = NULL;
static void vala_array_dispose (GObject * obj);


ValaArray* vala_array_new (ValaDataType* _element_type, int _rank)
{
	g_return_val_if_fail (_element_type == NULL || VALA_IS_DATA_TYPE (_element_type), NULL);
	GParameter * __params = g_new0 (GParameter, 2);
	GParameter * __params_it = __params;
	(__params_it->name = "rank", g_value_init (&__params_it->value, G_TYPE_INT), g_value_set_int (&__params_it->value, _rank), __params_it++);
	(__params_it->name = "element-type", g_value_init (&__params_it->value, VALA_TYPE_DATA_TYPE), g_value_set_object (&__params_it->value, _element_type), __params_it++);
	ValaArray * self = g_object_newv (VALA_TYPE_ARRAY, __params_it - __params, __params);
	if (_rank < 1) {
		vala_report_error (NULL, "internal: attempt to create an array with rank smaller than 1");
	}
	return self;
}


/**
 * Returns the name of this data type as it is used in C code.
 *
 * @return the name to be used in C code
 */
static char* vala_array_real_get_cname (ValaDataType* base, gboolean const_type)
{
	ValaArray * self = VALA_ARRAY (base);
	if (self->priv->cname == NULL) {
		if (vala_data_type_is_reference_type (vala_array_get_element_type (self))) {
			char* __temp0 = NULL;
			self->priv->cname = (__temp0 = g_strdup_printf ("%s*", vala_data_type_get_cname (vala_array_get_element_type (self), FALSE)), (self->priv->cname == NULL ? NULL : (self->priv->cname = (g_free (self->priv->cname), NULL))), __temp0);
		} else {
			char* __temp2 = NULL;
			const char* __temp1 = NULL;
			self->priv->cname = (__temp2 = (__temp1 = vala_data_type_get_cname (vala_array_get_element_type (self), FALSE), (__temp1 == NULL ? NULL : g_strdup (__temp1))), (self->priv->cname == NULL ? NULL : (self->priv->cname = (g_free (self->priv->cname), NULL))), __temp2);
		}
	}
	return self->priv->cname;
}


/**
 * Checks whether this data type has value or reference type semantics.
 *
 * @return true if this data type has reference type semantics
 */
static gboolean vala_array_real_is_reference_type (ValaDataType* base)
{
	ValaArray * self = VALA_ARRAY (base);
	return TRUE;
}


/**
 * Returns the C name of this data type in upper case. Words are
 * separated by underscores. The upper case C name of the namespace is
 * prefix of the result.
 *
 * @param infix a string to be placed between namespace and data type
 *              name or null
 * @return      the upper case name to be used in C code
 */
static char* vala_array_real_get_upper_case_cname (ValaDataType* base, char* infix)
{
	ValaArray * self = VALA_ARRAY (base);
	return NULL;
}


/**
 * Returns the C name of this data type in lower case. Words are
 * separated by underscores. The lower case C name of the namespace is
 * prefix of the result.
 *
 * @param infix a string to be placed between namespace and data type
 *              name or null
 * @return      the lower case name to be used in C code
 */
static char* vala_array_real_get_lower_case_cname (ValaDataType* base, char* infix)
{
	ValaArray * self = VALA_ARRAY (base);
	return NULL;
}


/**
 * Returns the C function name that frees instances of this data type.
 * This is only valid for data types with reference type semantics that
 * do not support reference counting. The specified C function must
 * accept one argument pointing to the instance to be freed.
 *
 * @return the name of the C function or null if this data type is not a
 *         reference type or if it supports reference counting
 */
static char* vala_array_real_get_free_function (ValaDataType* base)
{
	ValaArray * self = VALA_ARRAY (base);
	return "g_free";
}


/**
 * Returns a list of C header filenames users of this data type must
 * include.
 *
 * @return list of C header filenames for this data type
 */
static GList* vala_array_real_get_cheader_filenames (ValaDataType* base)
{
	ValaArray * self = VALA_ARRAY (base);
	return vala_data_type_get_cheader_filenames (vala_array_get_element_type (self));
}


static char* vala_array_real_get_marshaller_type_name (ValaDataType* base)
{
	ValaArray * self = VALA_ARRAY (base);
	return "POINTER";
}


static char* vala_array_real_get_get_value_function (ValaDataType* base)
{
	ValaArray * self = VALA_ARRAY (base);
	return "g_value_get_pointer";
}


static char* vala_array_real_get_set_value_function (ValaDataType* base)
{
	ValaArray * self = VALA_ARRAY (base);
	return "g_value_set_pointer";
}


ValaDataType* vala_array_get_element_type (ValaArray* self)
{
	g_return_val_if_fail (VALA_IS_ARRAY (self), NULL);
	return self->priv->_element_type;
}


void vala_array_set_element_type (ValaArray* self, ValaDataType* value)
{
	g_return_if_fail (VALA_IS_ARRAY (self));
	ValaDataType* __temp13 = NULL;
	self->priv->_element_type = (__temp13 = g_object_ref (value), (self->priv->_element_type == NULL ? NULL : (self->priv->_element_type = (g_object_unref (self->priv->_element_type), NULL))), __temp13);
}


int vala_array_get_rank (ValaArray* self)
{
	g_return_val_if_fail (VALA_IS_ARRAY (self), 0);
	return self->priv->_rank;
}


void vala_array_set_rank (ValaArray* self, int value)
{
	g_return_if_fail (VALA_IS_ARRAY (self));
	self->priv->_rank = value;
}


static GObject * vala_array_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties)
{
	GObject * obj;
	ValaArrayClass * klass;
	GObjectClass * parent_class;
	klass = VALA_ARRAY_CLASS (g_type_class_peek (VALA_TYPE_ARRAY));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	ValaArray * self = VALA_ARRAY (obj);
	{
		/* FIXME: this implementation raises compiler bugs 
		string commas = "";
		int i = rank - 1;
		
		while (i > 0) {
		string += ",";
		i--;
		}
		
		name = "%s[%s]".printf (element_type.name, commas); */
		int i = vala_array_get_rank (self) - 1;
		char* __temp15 = NULL;
		vala_data_type_set_name (VALA_DATA_TYPE (self), (__temp15 = g_strdup_printf ("%s[", vala_data_type_get_name (vala_array_get_element_type (self)))));
		(__temp15 == NULL ? NULL : (__temp15 = (g_free (__temp15), NULL)));
		while (i > 0) {
			char* __temp16 = NULL;
			vala_data_type_set_name (VALA_DATA_TYPE (self), (__temp16 = g_strdup_printf ("%s,", vala_data_type_get_name (VALA_DATA_TYPE (self)))));
			(__temp16 == NULL ? NULL : (__temp16 = (g_free (__temp16), NULL)));
			i--;
		}
		char* __temp17 = NULL;
		vala_data_type_set_name (VALA_DATA_TYPE (self), (__temp17 = g_strdup_printf ("%s]", vala_data_type_get_name (VALA_DATA_TYPE (self)))));
		(__temp17 == NULL ? NULL : (__temp17 = (g_free (__temp17), NULL)));
	}
	return obj;
}


static void vala_array_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaArray * self = VALA_ARRAY (object);
	switch (property_id) {
		case VALA_ARRAY_ELEMENT_TYPE:
		g_value_set_object (value, vala_array_get_element_type (self));
		break;
		case VALA_ARRAY_RANK:
		g_value_set_int (value, vala_array_get_rank (self));
		break;
	}
}


static void vala_array_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaArray * self = VALA_ARRAY (object);
	switch (property_id) {
		case VALA_ARRAY_ELEMENT_TYPE:
		vala_array_set_element_type (self, g_value_get_object (value));
		break;
		case VALA_ARRAY_RANK:
		vala_array_set_rank (self, g_value_get_int (value));
		break;
	}
}


static void vala_array_class_init (ValaArrayClass * klass)
{
	vala_array_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValaArrayPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_array_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_array_set_property;
	G_OBJECT_CLASS (klass)->constructor = vala_array_constructor;
	G_OBJECT_CLASS (klass)->dispose = vala_array_dispose;
	VALA_DATA_TYPE_CLASS (klass)->get_cname = vala_array_real_get_cname;
	VALA_DATA_TYPE_CLASS (klass)->is_reference_type = vala_array_real_is_reference_type;
	VALA_DATA_TYPE_CLASS (klass)->get_upper_case_cname = vala_array_real_get_upper_case_cname;
	VALA_DATA_TYPE_CLASS (klass)->get_lower_case_cname = vala_array_real_get_lower_case_cname;
	VALA_DATA_TYPE_CLASS (klass)->get_free_function = vala_array_real_get_free_function;
	VALA_DATA_TYPE_CLASS (klass)->get_cheader_filenames = vala_array_real_get_cheader_filenames;
	VALA_DATA_TYPE_CLASS (klass)->get_marshaller_type_name = vala_array_real_get_marshaller_type_name;
	VALA_DATA_TYPE_CLASS (klass)->get_get_value_function = vala_array_real_get_get_value_function;
	VALA_DATA_TYPE_CLASS (klass)->get_set_value_function = vala_array_real_get_set_value_function;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_ARRAY_ELEMENT_TYPE, g_param_spec_object ("element-type", "foo", "bar", VALA_TYPE_DATA_TYPE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_ARRAY_RANK, g_param_spec_int ("rank", "foo", "bar", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
}


static void vala_array_init (ValaArray * self)
{
	self->priv = VALA_ARRAY_GET_PRIVATE (self);
}


static void vala_array_dispose (GObject * obj)
{
	ValaArray * self = VALA_ARRAY (obj);
	(self->priv->_element_type == NULL ? NULL : (self->priv->_element_type = (g_object_unref (self->priv->_element_type), NULL)));
	(self->priv->cname == NULL ? NULL : (self->priv->cname = (g_free (self->priv->cname), NULL)));
	ValaArrayClass * klass;
	GObjectClass * parent_class;
	klass = VALA_ARRAY_CLASS (g_type_class_peek (VALA_TYPE_ARRAY));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_array_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaArrayClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_array_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaArray), 0, (GInstanceInitFunc) vala_array_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_DATA_TYPE, "ValaArray", &g_define_type_info, 0);
	}
	return g_define_type_id;
}




