/* valasymbolbuilder.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */
#define VALA_FREE_CHECKED(o,f) ((o) == NULL ? NULL : ((o) = (f (o), NULL)))
#define VALA_FREE_UNCHECKED(o,f) ((o) = (f (o), NULL))

#include "valasymbolbuilder.h"
#include <vala/valasymbolbuilder.h>
#include <vala/valareport.h>
#include <vala/valasourcereference.h>
#include <vala/valadatatype.h>
#include <vala/valatypereference.h>
#include <vala/valareturnstatement.h>
#include <vala/valamemberaccess.h>
#include <vala/valaexpression.h>
#include <vala/valastatement.h>
#include <vala/valaexpressionstatement.h>
#include <vala/valaassignment.h>

struct _ValaSymbolBuilderPrivate {
	ValaSymbol* root;
	ValaSymbol* current_type;
	ValaSymbol* current_symbol;
	ValaSourceFile* current_source_file;
};
#define VALA_SYMBOL_BUILDER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_SYMBOL_BUILDER, ValaSymbolBuilderPrivate))
enum  {
	VALA_SYMBOL_BUILDER_DUMMY_PROPERTY,
};
static void vala_symbol_builder_real_visit_begin_source_file (ValaCodeVisitor* base, ValaSourceFile* file);
static void vala_symbol_builder_real_visit_begin_namespace (ValaCodeVisitor* base, ValaNamespace* ns);
static void vala_symbol_builder_real_visit_end_namespace (ValaCodeVisitor* base, ValaNamespace* ns);
static ValaSymbol* vala_symbol_builder_add_symbol (ValaSymbolBuilder* self, char* name, ValaCodeNode* node);
static void vala_symbol_builder_real_visit_begin_class (ValaCodeVisitor* base, ValaClass* cl);
static void vala_symbol_builder_real_visit_end_class (ValaCodeVisitor* base, ValaClass* cl);
static void vala_symbol_builder_real_visit_begin_struct (ValaCodeVisitor* base, ValaStruct* st);
static void vala_symbol_builder_real_visit_end_struct (ValaCodeVisitor* base, ValaStruct* st);
static void vala_symbol_builder_real_visit_begin_interface (ValaCodeVisitor* base, ValaInterface* iface);
static void vala_symbol_builder_real_visit_end_interface (ValaCodeVisitor* base, ValaInterface* iface);
static void vala_symbol_builder_real_visit_begin_enum (ValaCodeVisitor* base, ValaEnum* en);
static void vala_symbol_builder_real_visit_end_enum (ValaCodeVisitor* base, ValaEnum* en);
static void vala_symbol_builder_real_visit_enum_value (ValaCodeVisitor* base, ValaEnumValue* ev);
static void vala_symbol_builder_real_visit_begin_callback (ValaCodeVisitor* base, ValaCallback* cb);
static void vala_symbol_builder_real_visit_end_callback (ValaCodeVisitor* base, ValaCallback* cb);
static void vala_symbol_builder_real_visit_constant (ValaCodeVisitor* base, ValaConstant* c);
static void vala_symbol_builder_real_visit_field (ValaCodeVisitor* base, ValaField* f);
static void vala_symbol_builder_real_visit_begin_method (ValaCodeVisitor* base, ValaMethod* m);
static void vala_symbol_builder_real_visit_end_method (ValaCodeVisitor* base, ValaMethod* m);
static void vala_symbol_builder_real_visit_formal_parameter (ValaCodeVisitor* base, ValaFormalParameter* p);
static void vala_symbol_builder_real_visit_begin_property (ValaCodeVisitor* base, ValaProperty* prop);
static void vala_symbol_builder_real_visit_end_property (ValaCodeVisitor* base, ValaProperty* prop);
static void vala_symbol_builder_real_visit_begin_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc);
static void vala_symbol_builder_real_visit_end_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc);
static void vala_symbol_builder_real_visit_begin_signal (ValaCodeVisitor* base, ValaSignal* sig);
static void vala_symbol_builder_real_visit_end_signal (ValaCodeVisitor* base, ValaSignal* sig);
static void vala_symbol_builder_real_visit_begin_constructor (ValaCodeVisitor* base, ValaConstructor* c);
static void vala_symbol_builder_real_visit_end_constructor (ValaCodeVisitor* base, ValaConstructor* c);
static void vala_symbol_builder_real_visit_begin_destructor (ValaCodeVisitor* base, ValaDestructor* d);
static void vala_symbol_builder_real_visit_end_destructor (ValaCodeVisitor* base, ValaDestructor* d);
static void vala_symbol_builder_real_visit_begin_block (ValaCodeVisitor* base, ValaBlock* b);
static void vala_symbol_builder_real_visit_end_block (ValaCodeVisitor* base, ValaBlock* b);
static void vala_symbol_builder_real_visit_type_parameter (ValaCodeVisitor* base, ValaTypeParameter* p);
static gpointer vala_symbol_builder_parent_class = NULL;
static void vala_symbol_builder_dispose (GObject * obj);


/**
 * Build the symbol tree for the specified code context.
 *
 * @param context a code context
 */
void vala_symbol_builder_build (ValaSymbolBuilder* self, ValaCodeContext* context)
{
	g_return_if_fail (VALA_IS_SYMBOL_BUILDER (self));
	g_return_if_fail (VALA_IS_CODE_CONTEXT (context));
	ValaSymbol* __temp0 = NULL;
	self->priv->root = (__temp0 = g_object_ref (vala_code_context_get_root (context)), (self->priv->root == NULL ? NULL : (self->priv->root = (g_object_unref (self->priv->root), NULL))), __temp0);
	vala_code_context_accept (context, VALA_CODE_VISITOR (self));
}


static void vala_symbol_builder_real_visit_begin_source_file (ValaCodeVisitor* base, ValaSourceFile* file)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_SOURCE_FILE (file));
	ValaSourceFile* __temp1 = NULL;
	self->priv->current_source_file = (__temp1 = g_object_ref (file), (self->priv->current_source_file == NULL ? NULL : (self->priv->current_source_file = (g_object_unref (self->priv->current_source_file), NULL))), __temp1);
}


static void vala_symbol_builder_real_visit_begin_namespace (ValaCodeVisitor* base, ValaNamespace* ns)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_NAMESPACE (ns));
	if (vala_namespace_get_name (ns) == NULL) {
		vala_code_node_set_symbol (VALA_CODE_NODE (ns), self->priv->root);
	}
	if (vala_code_node_get_symbol (VALA_CODE_NODE (ns)) == NULL) {
		vala_code_node_set_symbol (VALA_CODE_NODE (ns), vala_symbol_lookup (self->priv->root, vala_namespace_get_name (ns)));
	}
	if (vala_code_node_get_symbol (VALA_CODE_NODE (ns)) == NULL) {
		ValaSymbol* __temp2 = NULL;
		vala_code_node_set_symbol (VALA_CODE_NODE (ns), (__temp2 = vala_symbol_new (VALA_CODE_NODE (ns))));
		(__temp2 == NULL ? NULL : (__temp2 = (g_object_unref (__temp2), NULL)));
		vala_symbol_add (self->priv->root, vala_namespace_get_name (ns), vala_code_node_get_symbol (VALA_CODE_NODE (ns)));
	}
	ValaSymbol* __temp4 = NULL;
	ValaSymbol* __temp3 = NULL;
	self->priv->current_symbol = (__temp4 = (__temp3 = vala_code_node_get_symbol (VALA_CODE_NODE (ns)), (__temp3 == NULL ? NULL : g_object_ref (__temp3))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp4);
}


static void vala_symbol_builder_real_visit_end_namespace (ValaCodeVisitor* base, ValaNamespace* ns)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_NAMESPACE (ns));
	ValaSymbol* __temp6 = NULL;
	ValaSymbol* __temp5 = NULL;
	self->priv->current_symbol = (__temp6 = (__temp5 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp5 == NULL ? NULL : g_object_ref (__temp5))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp6);
}


static ValaSymbol* vala_symbol_builder_add_symbol (ValaSymbolBuilder* self, char* name, ValaCodeNode* node)
{
	g_return_val_if_fail (VALA_IS_SYMBOL_BUILDER (self), NULL);
	g_return_val_if_fail (VALA_IS_CODE_NODE (node), NULL);
	if (name != NULL) {
		if (vala_symbol_lookup (self->priv->current_symbol, name) != NULL) {
			vala_code_node_set_error (node, TRUE);
			char* __temp8 = NULL;
			char* __temp7 = NULL;
			vala_report_error (vala_code_node_get_source_reference (node), (__temp8 = g_strdup_printf ("`%s' already contains a definition for `%s'", (__temp7 = vala_symbol_get_full_name (self->priv->current_symbol)), name)));
			(__temp8 == NULL ? NULL : (__temp8 = (g_free (__temp8), NULL)));
			(__temp7 == NULL ? NULL : (__temp7 = (g_free (__temp7), NULL)));
			return VALA_SYMBOL (NULL);
		}
	}
	ValaSymbol* __temp10 = NULL;
	vala_code_node_set_symbol (node, (__temp10 = vala_symbol_new (node)));
	(__temp10 == NULL ? NULL : (__temp10 = (g_object_unref (__temp10), NULL)));
	if (name != NULL) {
		vala_symbol_add (self->priv->current_symbol, name, vala_code_node_get_symbol (node));
	} else {
		vala_symbol_set_parent_symbol (vala_code_node_get_symbol (node), self->priv->current_symbol);
	}
	return vala_code_node_get_symbol (node);
}


static void vala_symbol_builder_real_visit_begin_class (ValaCodeVisitor* base, ValaClass* cl)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_CLASS (cl));
	if (vala_symbol_builder_add_symbol (self, vala_data_type_get_name (VALA_DATA_TYPE (cl)), VALA_CODE_NODE (cl)) == NULL) {
		return;
	}
	ValaSymbol* __temp13 = NULL;
	ValaSymbol* __temp12 = NULL;
	self->priv->current_symbol = (__temp13 = (__temp12 = vala_code_node_get_symbol (VALA_CODE_NODE (cl)), (__temp12 == NULL ? NULL : g_object_ref (__temp12))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp13);
}


static void vala_symbol_builder_real_visit_end_class (ValaCodeVisitor* base, ValaClass* cl)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_CLASS (cl));
	if (vala_code_node_get_error (VALA_CODE_NODE (cl))) {
		return;
	}
	/* skip classes with errors */
	ValaSymbol* __temp15 = NULL;
	ValaSymbol* __temp14 = NULL;
	self->priv->current_symbol = (__temp15 = (__temp14 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp14 == NULL ? NULL : g_object_ref (__temp14))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp15);
}


static void vala_symbol_builder_real_visit_begin_struct (ValaCodeVisitor* base, ValaStruct* st)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_STRUCT (st));
	if (vala_symbol_builder_add_symbol (self, vala_data_type_get_name (VALA_DATA_TYPE (st)), VALA_CODE_NODE (st)) == NULL) {
		return;
	}
	ValaSymbol* __temp17 = NULL;
	ValaSymbol* __temp16 = NULL;
	self->priv->current_symbol = (__temp17 = (__temp16 = vala_code_node_get_symbol (VALA_CODE_NODE (st)), (__temp16 == NULL ? NULL : g_object_ref (__temp16))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp17);
}


static void vala_symbol_builder_real_visit_end_struct (ValaCodeVisitor* base, ValaStruct* st)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_STRUCT (st));
	if (vala_code_node_get_error (VALA_CODE_NODE (st))) {
		return;
	}
	/* skip structs with errors */
	ValaSymbol* __temp19 = NULL;
	ValaSymbol* __temp18 = NULL;
	self->priv->current_symbol = (__temp19 = (__temp18 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp18 == NULL ? NULL : g_object_ref (__temp18))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp19);
}


static void vala_symbol_builder_real_visit_begin_interface (ValaCodeVisitor* base, ValaInterface* iface)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_INTERFACE (iface));
	if (vala_symbol_builder_add_symbol (self, vala_data_type_get_name (VALA_DATA_TYPE (iface)), VALA_CODE_NODE (iface)) == NULL) {
		return;
	}
	ValaSymbol* __temp21 = NULL;
	ValaSymbol* __temp20 = NULL;
	self->priv->current_symbol = (__temp21 = (__temp20 = vala_code_node_get_symbol (VALA_CODE_NODE (iface)), (__temp20 == NULL ? NULL : g_object_ref (__temp20))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp21);
}


static void vala_symbol_builder_real_visit_end_interface (ValaCodeVisitor* base, ValaInterface* iface)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_INTERFACE (iface));
	if (vala_code_node_get_error (VALA_CODE_NODE (iface))) {
		return;
	}
	/* skip interfaces with errors */
	ValaSymbol* __temp23 = NULL;
	ValaSymbol* __temp22 = NULL;
	self->priv->current_symbol = (__temp23 = (__temp22 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp22 == NULL ? NULL : g_object_ref (__temp22))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp23);
}


static void vala_symbol_builder_real_visit_begin_enum (ValaCodeVisitor* base, ValaEnum* en)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_ENUM (en));
	if (vala_symbol_builder_add_symbol (self, vala_data_type_get_name (VALA_DATA_TYPE (en)), VALA_CODE_NODE (en)) == NULL) {
		return;
	}
	ValaSymbol* __temp25 = NULL;
	ValaSymbol* __temp24 = NULL;
	self->priv->current_symbol = (__temp25 = (__temp24 = vala_code_node_get_symbol (VALA_CODE_NODE (en)), (__temp24 == NULL ? NULL : g_object_ref (__temp24))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp25);
}


static void vala_symbol_builder_real_visit_end_enum (ValaCodeVisitor* base, ValaEnum* en)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_ENUM (en));
	if (vala_code_node_get_error (VALA_CODE_NODE (en))) {
		return;
	}
	/* skip enums with errors */
	ValaSymbol* __temp27 = NULL;
	ValaSymbol* __temp26 = NULL;
	self->priv->current_symbol = (__temp27 = (__temp26 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp26 == NULL ? NULL : g_object_ref (__temp26))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp27);
}


static void vala_symbol_builder_real_visit_enum_value (ValaCodeVisitor* base, ValaEnumValue* ev)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_ENUM_VALUE (ev));
	ValaSymbol* __temp28 = NULL;
	vala_code_node_set_symbol (VALA_CODE_NODE (ev), (__temp28 = vala_symbol_new (VALA_CODE_NODE (ev))));
	(__temp28 == NULL ? NULL : (__temp28 = (g_object_unref (__temp28), NULL)));
	vala_symbol_add (self->priv->current_symbol, vala_enum_value_get_name (ev), vala_code_node_get_symbol (VALA_CODE_NODE (ev)));
}


static void vala_symbol_builder_real_visit_begin_callback (ValaCodeVisitor* base, ValaCallback* cb)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_CALLBACK (cb));
	if (vala_symbol_builder_add_symbol (self, vala_data_type_get_name (VALA_DATA_TYPE (cb)), VALA_CODE_NODE (cb)) == NULL) {
		return;
	}
	ValaSymbol* __temp30 = NULL;
	ValaSymbol* __temp29 = NULL;
	self->priv->current_symbol = (__temp30 = (__temp29 = vala_code_node_get_symbol (VALA_CODE_NODE (cb)), (__temp29 == NULL ? NULL : g_object_ref (__temp29))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp30);
}


static void vala_symbol_builder_real_visit_end_callback (ValaCodeVisitor* base, ValaCallback* cb)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_CALLBACK (cb));
	if (vala_code_node_get_error (VALA_CODE_NODE (cb))) {
		return;
	}
	/* skip enums with errors */
	ValaSymbol* __temp32 = NULL;
	ValaSymbol* __temp31 = NULL;
	self->priv->current_symbol = (__temp32 = (__temp31 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp31 == NULL ? NULL : g_object_ref (__temp31))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp32);
}


static void vala_symbol_builder_real_visit_constant (ValaCodeVisitor* base, ValaConstant* c)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_CONSTANT (c));
	vala_symbol_builder_add_symbol (self, vala_constant_get_name (c), VALA_CODE_NODE (c));
}


static void vala_symbol_builder_real_visit_field (ValaCodeVisitor* base, ValaField* f)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_FIELD (f));
	vala_symbol_builder_add_symbol (self, vala_field_get_name (f), VALA_CODE_NODE (f));
}


static void vala_symbol_builder_real_visit_begin_method (ValaCodeVisitor* base, ValaMethod* m)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_METHOD (m));
	if (vala_symbol_builder_add_symbol (self, vala_method_get_name (m), VALA_CODE_NODE (m)) == NULL) {
		return;
	}
	if (vala_method_get_construction (m)) {
		ValaCodeNode* type_node = vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (m))));
		if (!(VALA_IS_CLASS (type_node) || VALA_IS_STRUCT (type_node))) {
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (m)), "construction methods may only be declared within classes and structs");
			vala_code_node_set_error (VALA_CODE_NODE (m), TRUE);
			return;
		}
		if (vala_method_get_name (m) == NULL) {
			if (VALA_IS_CLASS (type_node)) {
				vala_class_set_default_construction_method ((VALA_CLASS (type_node)), m);
			} else {
				if (VALA_IS_STRUCT (type_node)) {
					vala_struct_set_default_construction_method ((VALA_STRUCT (type_node)), m);
				}
			}
		}
	} else {
		if (vala_method_get_instance (m)) {
			if (!(VALA_IS_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (m))))))) {
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (m)), "instance methods not allowed outside of data types");
				vala_code_node_set_error (VALA_CODE_NODE (m), TRUE);
				return;
			}
			ValaFormalParameter* __temp33 = NULL;
			vala_method_set_this_parameter (m, (__temp33 = vala_formal_parameter_new ("this", vala_type_reference_new (), NULL)));
			(__temp33 == NULL ? NULL : (__temp33 = (g_object_unref (__temp33), NULL)));
			vala_type_reference_set_data_type (vala_formal_parameter_get_type_reference (vala_method_get_this_parameter (m)), VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (m))))));
			ValaSymbol* __temp34 = NULL;
			vala_code_node_set_symbol (VALA_CODE_NODE (vala_method_get_this_parameter (m)), (__temp34 = vala_symbol_new (VALA_CODE_NODE (vala_method_get_this_parameter (m)))));
			(__temp34 == NULL ? NULL : (__temp34 = (g_object_unref (__temp34), NULL)));
			vala_symbol_add (self->priv->current_symbol, vala_formal_parameter_get_name (vala_method_get_this_parameter (m)), vala_code_node_get_symbol (VALA_CODE_NODE (vala_method_get_this_parameter (m))));
		}
	}
	ValaSymbol* __temp36 = NULL;
	ValaSymbol* __temp35 = NULL;
	self->priv->current_symbol = (__temp36 = (__temp35 = vala_code_node_get_symbol (VALA_CODE_NODE (m)), (__temp35 == NULL ? NULL : g_object_ref (__temp35))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp36);
}


static void vala_symbol_builder_real_visit_end_method (ValaCodeVisitor* base, ValaMethod* m)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_METHOD (m));
	if (vala_code_node_get_error (VALA_CODE_NODE (m))) {
		return;
	}
	/* skip methods with errors */
	ValaSymbol* __temp38 = NULL;
	ValaSymbol* __temp37 = NULL;
	self->priv->current_symbol = (__temp38 = (__temp37 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp37 == NULL ? NULL : g_object_ref (__temp37))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp38);
}


static void vala_symbol_builder_real_visit_formal_parameter (ValaCodeVisitor* base, ValaFormalParameter* p)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_FORMAL_PARAMETER (p));
	if (!vala_formal_parameter_get_ellipsis (p)) {
		vala_symbol_builder_add_symbol (self, vala_formal_parameter_get_name (p), VALA_CODE_NODE (p));
	}
}


static void vala_symbol_builder_real_visit_begin_property (ValaCodeVisitor* base, ValaProperty* prop)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_PROPERTY (prop));
	if (vala_symbol_builder_add_symbol (self, vala_property_get_name (prop), VALA_CODE_NODE (prop)) == NULL) {
		return;
	}
	ValaSymbol* __temp40 = NULL;
	ValaSymbol* __temp39 = NULL;
	self->priv->current_symbol = (__temp40 = (__temp39 = vala_code_node_get_symbol (VALA_CODE_NODE (prop)), (__temp39 == NULL ? NULL : g_object_ref (__temp39))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp40);
	ValaFormalParameter* __temp41 = NULL;
	vala_property_set_this_parameter (prop, (__temp41 = vala_formal_parameter_new ("this", vala_type_reference_new (), NULL)));
	(__temp41 == NULL ? NULL : (__temp41 = (g_object_unref (__temp41), NULL)));
	vala_type_reference_set_data_type (vala_formal_parameter_get_type_reference (vala_property_get_this_parameter (prop)), VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (prop))))));
	ValaSymbol* __temp42 = NULL;
	vala_code_node_set_symbol (VALA_CODE_NODE (vala_property_get_this_parameter (prop)), (__temp42 = vala_symbol_new (VALA_CODE_NODE (vala_property_get_this_parameter (prop)))));
	(__temp42 == NULL ? NULL : (__temp42 = (g_object_unref (__temp42), NULL)));
	vala_symbol_add (self->priv->current_symbol, vala_formal_parameter_get_name (vala_property_get_this_parameter (prop)), vala_code_node_get_symbol (VALA_CODE_NODE (vala_property_get_this_parameter (prop))));
}


static void vala_symbol_builder_real_visit_end_property (ValaCodeVisitor* base, ValaProperty* prop)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_PROPERTY (prop));
	if (vala_code_node_get_error (VALA_CODE_NODE (prop))) {
		return;
	}
	/* skip properties with errors */
	ValaSymbol* __temp44 = NULL;
	ValaSymbol* __temp43 = NULL;
	self->priv->current_symbol = (__temp44 = (__temp43 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp43 == NULL ? NULL : g_object_ref (__temp43))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp44);
}


static void vala_symbol_builder_real_visit_begin_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_PROPERTY_ACCESSOR (acc));
	ValaSymbol* __temp45 = NULL;
	vala_code_node_set_symbol (VALA_CODE_NODE (acc), (__temp45 = vala_symbol_new (VALA_CODE_NODE (acc))));
	(__temp45 == NULL ? NULL : (__temp45 = (g_object_unref (__temp45), NULL)));
	vala_symbol_set_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (acc)), self->priv->current_symbol);
	ValaSymbol* __temp47 = NULL;
	ValaSymbol* __temp46 = NULL;
	self->priv->current_symbol = (__temp47 = (__temp46 = vala_code_node_get_symbol (VALA_CODE_NODE (acc)), (__temp46 == NULL ? NULL : g_object_ref (__temp46))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp47);
	if (vala_source_file_get_pkg (self->priv->current_source_file)) {
		return;
	}
	if (vala_property_accessor_get_writable (acc) || vala_property_accessor_get_construction (acc)) {
		ValaFormalParameter* __temp48 = NULL;
		vala_property_accessor_set_value_parameter (acc, (__temp48 = vala_formal_parameter_new ("value", vala_property_get_type_reference ((VALA_PROPERTY (vala_symbol_get_node (vala_symbol_get_parent_symbol (self->priv->current_symbol))))), NULL)));
		(__temp48 == NULL ? NULL : (__temp48 = (g_object_unref (__temp48), NULL)));
		ValaSymbol* __temp49 = NULL;
		vala_code_node_set_symbol (VALA_CODE_NODE (vala_property_accessor_get_value_parameter (acc)), (__temp49 = vala_symbol_new (VALA_CODE_NODE (vala_property_accessor_get_value_parameter (acc)))));
		(__temp49 == NULL ? NULL : (__temp49 = (g_object_unref (__temp49), NULL)));
		vala_symbol_add (self->priv->current_symbol, vala_formal_parameter_get_name (vala_property_accessor_get_value_parameter (acc)), vala_code_node_get_symbol (VALA_CODE_NODE (vala_property_accessor_get_value_parameter (acc))));
	}
	if (vala_property_accessor_get_body (acc) == NULL) {
		/* no accessor body specified, insert default body */
		ValaProperty* prop = VALA_PROPERTY (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (acc)))));
		if (vala_property_get_interface_only (prop)) {
			return;
		}
		ValaBlock* block = vala_block_new (NULL);
		if (vala_property_accessor_get_readable (acc)) {
			ValaReturnStatement* __temp50 = NULL;
			vala_block_add_statement (block, VALA_STATEMENT ((__temp50 = vala_return_statement_new (VALA_EXPRESSION (vala_member_access_new_simple (g_strdup_printf ("_%s", vala_property_get_name (prop)), NULL)), NULL))));
			(__temp50 == NULL ? NULL : (__temp50 = (g_object_unref (__temp50), NULL)));
		} else {
			ValaExpressionStatement* __temp51 = NULL;
			vala_block_add_statement (block, VALA_STATEMENT ((__temp51 = vala_expression_statement_new (VALA_EXPRESSION (vala_assignment_new (VALA_EXPRESSION (vala_member_access_new_simple (g_strdup_printf ("_%s", vala_property_get_name (prop)), NULL)), VALA_EXPRESSION (vala_member_access_new_simple ("value", NULL)), VALA_ASSIGNMENT_OPERATOR_SIMPLE, NULL)), NULL))));
			(__temp51 == NULL ? NULL : (__temp51 = (g_object_unref (__temp51), NULL)));
		}
		vala_property_accessor_set_body (acc, VALA_STATEMENT (block));
		(block == NULL ? NULL : (block = (g_object_unref (block), NULL)));
	}
}


static void vala_symbol_builder_real_visit_end_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_PROPERTY_ACCESSOR (acc));
	ValaSymbol* __temp53 = NULL;
	ValaSymbol* __temp52 = NULL;
	self->priv->current_symbol = (__temp53 = (__temp52 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp52 == NULL ? NULL : g_object_ref (__temp52))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp53);
}


static void vala_symbol_builder_real_visit_begin_signal (ValaCodeVisitor* base, ValaSignal* sig)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_SIGNAL (sig));
	if (vala_symbol_builder_add_symbol (self, vala_signal_get_name (sig), VALA_CODE_NODE (sig)) == NULL) {
		return;
	}
	ValaSymbol* __temp55 = NULL;
	ValaSymbol* __temp54 = NULL;
	self->priv->current_symbol = (__temp55 = (__temp54 = vala_code_node_get_symbol (VALA_CODE_NODE (sig)), (__temp54 == NULL ? NULL : g_object_ref (__temp54))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp55);
}


static void vala_symbol_builder_real_visit_end_signal (ValaCodeVisitor* base, ValaSignal* sig)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_SIGNAL (sig));
	if (vala_code_node_get_error (VALA_CODE_NODE (sig))) {
		return;
	}
	/* skip signals with errors */
	ValaSymbol* __temp57 = NULL;
	ValaSymbol* __temp56 = NULL;
	self->priv->current_symbol = (__temp57 = (__temp56 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp56 == NULL ? NULL : g_object_ref (__temp56))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp57);
}


static void vala_symbol_builder_real_visit_begin_constructor (ValaCodeVisitor* base, ValaConstructor* c)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_CONSTRUCTOR (c));
	ValaSymbol* __temp58 = NULL;
	vala_code_node_set_symbol (VALA_CODE_NODE (c), (__temp58 = vala_symbol_new (VALA_CODE_NODE (c))));
	(__temp58 == NULL ? NULL : (__temp58 = (g_object_unref (__temp58), NULL)));
	vala_symbol_set_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (c)), self->priv->current_symbol);
	ValaSymbol* __temp60 = NULL;
	ValaSymbol* __temp59 = NULL;
	self->priv->current_symbol = (__temp60 = (__temp59 = vala_code_node_get_symbol (VALA_CODE_NODE (c)), (__temp59 == NULL ? NULL : g_object_ref (__temp59))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp60);
}


static void vala_symbol_builder_real_visit_end_constructor (ValaCodeVisitor* base, ValaConstructor* c)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_CONSTRUCTOR (c));
	ValaSymbol* __temp62 = NULL;
	ValaSymbol* __temp61 = NULL;
	self->priv->current_symbol = (__temp62 = (__temp61 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp61 == NULL ? NULL : g_object_ref (__temp61))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp62);
}


static void vala_symbol_builder_real_visit_begin_destructor (ValaCodeVisitor* base, ValaDestructor* d)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_DESTRUCTOR (d));
	ValaSymbol* __temp63 = NULL;
	vala_code_node_set_symbol (VALA_CODE_NODE (d), (__temp63 = vala_symbol_new (VALA_CODE_NODE (d))));
	(__temp63 == NULL ? NULL : (__temp63 = (g_object_unref (__temp63), NULL)));
	vala_symbol_set_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (d)), self->priv->current_symbol);
	ValaSymbol* __temp65 = NULL;
	ValaSymbol* __temp64 = NULL;
	self->priv->current_symbol = (__temp65 = (__temp64 = vala_code_node_get_symbol (VALA_CODE_NODE (d)), (__temp64 == NULL ? NULL : g_object_ref (__temp64))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp65);
}


static void vala_symbol_builder_real_visit_end_destructor (ValaCodeVisitor* base, ValaDestructor* d)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_DESTRUCTOR (d));
	ValaSymbol* __temp67 = NULL;
	ValaSymbol* __temp66 = NULL;
	self->priv->current_symbol = (__temp67 = (__temp66 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp66 == NULL ? NULL : g_object_ref (__temp66))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp67);
}


static void vala_symbol_builder_real_visit_begin_block (ValaCodeVisitor* base, ValaBlock* b)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_BLOCK (b));
	ValaSymbol* __temp68 = NULL;
	vala_code_node_set_symbol (VALA_CODE_NODE (b), (__temp68 = vala_symbol_new (VALA_CODE_NODE (b))));
	(__temp68 == NULL ? NULL : (__temp68 = (g_object_unref (__temp68), NULL)));
	vala_symbol_set_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (b)), self->priv->current_symbol);
	ValaSymbol* __temp70 = NULL;
	ValaSymbol* __temp69 = NULL;
	self->priv->current_symbol = (__temp70 = (__temp69 = vala_code_node_get_symbol (VALA_CODE_NODE (b)), (__temp69 == NULL ? NULL : g_object_ref (__temp69))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp70);
}


static void vala_symbol_builder_real_visit_end_block (ValaCodeVisitor* base, ValaBlock* b)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_BLOCK (b));
	ValaSymbol* __temp72 = NULL;
	ValaSymbol* __temp71 = NULL;
	self->priv->current_symbol = (__temp72 = (__temp71 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp71 == NULL ? NULL : g_object_ref (__temp71))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp72);
}


static void vala_symbol_builder_real_visit_type_parameter (ValaCodeVisitor* base, ValaTypeParameter* p)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_TYPE_PARAMETER (p));
	vala_symbol_builder_add_symbol (self, vala_type_parameter_get_name (p), VALA_CODE_NODE (p));
}


static void vala_symbol_builder_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (object);
	switch (property_id) {
	}
}


static void vala_symbol_builder_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (object);
	switch (property_id) {
	}
}


static void vala_symbol_builder_class_init (ValaSymbolBuilderClass * klass)
{
	vala_symbol_builder_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValaSymbolBuilderPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_symbol_builder_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_symbol_builder_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_symbol_builder_dispose;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_source_file = vala_symbol_builder_real_visit_begin_source_file;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_namespace = vala_symbol_builder_real_visit_begin_namespace;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_namespace = vala_symbol_builder_real_visit_end_namespace;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_class = vala_symbol_builder_real_visit_begin_class;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_class = vala_symbol_builder_real_visit_end_class;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_struct = vala_symbol_builder_real_visit_begin_struct;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_struct = vala_symbol_builder_real_visit_end_struct;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_interface = vala_symbol_builder_real_visit_begin_interface;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_interface = vala_symbol_builder_real_visit_end_interface;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_enum = vala_symbol_builder_real_visit_begin_enum;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_enum = vala_symbol_builder_real_visit_end_enum;
	VALA_CODE_VISITOR_CLASS (klass)->visit_enum_value = vala_symbol_builder_real_visit_enum_value;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_callback = vala_symbol_builder_real_visit_begin_callback;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_callback = vala_symbol_builder_real_visit_end_callback;
	VALA_CODE_VISITOR_CLASS (klass)->visit_constant = vala_symbol_builder_real_visit_constant;
	VALA_CODE_VISITOR_CLASS (klass)->visit_field = vala_symbol_builder_real_visit_field;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_method = vala_symbol_builder_real_visit_begin_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_method = vala_symbol_builder_real_visit_end_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_formal_parameter = vala_symbol_builder_real_visit_formal_parameter;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_property = vala_symbol_builder_real_visit_begin_property;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_property = vala_symbol_builder_real_visit_end_property;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_property_accessor = vala_symbol_builder_real_visit_begin_property_accessor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_property_accessor = vala_symbol_builder_real_visit_end_property_accessor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_signal = vala_symbol_builder_real_visit_begin_signal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_signal = vala_symbol_builder_real_visit_end_signal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_constructor = vala_symbol_builder_real_visit_begin_constructor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_constructor = vala_symbol_builder_real_visit_end_constructor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_destructor = vala_symbol_builder_real_visit_begin_destructor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_destructor = vala_symbol_builder_real_visit_end_destructor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_block = vala_symbol_builder_real_visit_begin_block;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_block = vala_symbol_builder_real_visit_end_block;
	VALA_CODE_VISITOR_CLASS (klass)->visit_type_parameter = vala_symbol_builder_real_visit_type_parameter;
}


static void vala_symbol_builder_init (ValaSymbolBuilder * self)
{
	self->priv = VALA_SYMBOL_BUILDER_GET_PRIVATE (self);
}


static void vala_symbol_builder_dispose (GObject * obj)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (obj);
	(self->priv->root == NULL ? NULL : (self->priv->root = (g_object_unref (self->priv->root), NULL)));
	(self->priv->current_type == NULL ? NULL : (self->priv->current_type = (g_object_unref (self->priv->current_type), NULL)));
	(self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL)));
	(self->priv->current_source_file == NULL ? NULL : (self->priv->current_source_file = (g_object_unref (self->priv->current_source_file), NULL)));
	ValaSymbolBuilderClass * klass;
	GObjectClass * parent_class;
	klass = VALA_SYMBOL_BUILDER_CLASS (g_type_class_peek (VALA_TYPE_SYMBOL_BUILDER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_symbol_builder_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaSymbolBuilderClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_symbol_builder_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaSymbolBuilder), 0, (GInstanceInitFunc) vala_symbol_builder_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CODE_VISITOR, "ValaSymbolBuilder", &g_define_type_info, 0);
	}
	return g_define_type_id;
}




